package cn.wise.sc.cement.business.util;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.formula.functions.T;
import org.apache.poi.ss.usermodel.*;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 校核计算工具类
 * 
 * @author zhutianwei
 *
 */
public class CheckCountUtil {

	//倍数
	public final static BigDecimal param10 = new BigDecimal(10.00);
	public final static BigDecimal param100 = new BigDecimal(100.00);
	public final static BigDecimal param1000 = new BigDecimal(1000.00);
	public final static BigDecimal param10000 = new BigDecimal(10000.00);

	//object 转成  BigDecimal
	public static BigDecimal getBigDecimal(Object value ) {
		BigDecimal ret = null;
		if( value != null ) {
			if( value instanceof BigDecimal ) {
				ret = (BigDecimal) value;
			} else if( value instanceof String ) {
				ret = new BigDecimal( (String) value );
			} else if( value instanceof BigInteger) {
				ret = new BigDecimal( (BigInteger) value );
			} else if( value instanceof Number ) {
				ret = new BigDecimal( ((Number)value).doubleValue() );
			} else {
				throw new ClassCastException("Not possible to coerce ["+value+"] from class "+value.getClass()+" into a BigDecimal.");
			}
		}
		return ret;
	}

	/**
	 * 检测项名称
	 * @param name
	 * @return
	 */
	//加 add 减 subtract  乘 multiply  除 divide
	public static BigDecimal checkCount(String name, Map<String, Object> resultMap) {
		BigDecimal endResult = null;
		if(name.equals("L.O.I")){
			//样重m1—保留4位⼩数
			BigDecimal m1 = getBigDecimal(resultMap.get("样重m1")).setScale(4,BigDecimal.ROUND_HALF_UP);
			//空堝重m2—保留4位⼩数
			BigDecimal m2 = getBigDecimal(resultMap.get("空堝重m2")).setScale(4,BigDecimal.ROUND_HALF_UP);
			//烧后重m3—保留4位⼩数
			BigDecimal m3 = getBigDecimal(resultMap.get("烧后重m3")).setScale(4,BigDecimal.ROUND_HALF_UP);
			//计算的核心公式结果
			BigDecimal countResult = (m1.add(m2).subtract(m3)).divide(m1);
			BigDecimal result1 = countResult.multiply(param100.multiply(param100));
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.multiply(param100).setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.multiply(param100).subtract(new BigDecimal(0.0000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("SiO2")){
			//毫升数
			BigDecimal VSiO2 = getBigDecimal(resultMap.get("VSiO2"));
			//滴定度
			BigDecimal TSiO2 = getBigDecimal(resultMap.get("TSiO2"));
			//样m
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult = VSiO2.multiply(TSiO2).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.0000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("Al2O3+TiO2")){
			BigDecimal TAl2O3 = getBigDecimal(resultMap.get("TAl2O3"));
			BigDecimal VAl2O3 = getBigDecimal(resultMap.get("VAl2O3"));
			BigDecimal K = getBigDecimal(resultMap.get("K"));
			BigDecimal VCuSO4_I = getBigDecimal(resultMap.get("VCuSO4(I)"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult = TAl2O3.multiply(VAl2O3.subtract(K.multiply(VCuSO4_I))).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.0000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("Al2O3")){
			BigDecimal TAl2O3 = getBigDecimal(resultMap.get("TAl2O3"));
			BigDecimal VAl2O3 = getBigDecimal(resultMap.get("VAl2O3"));
			BigDecimal K = getBigDecimal(resultMap.get("K"));
			BigDecimal VCuSO4_I = getBigDecimal(resultMap.get("VCuSO4(I)"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			BigDecimal VCuSO4_II = getBigDecimal(resultMap.get("VCuSO4(II)"));
			//计算的核心公式结果
			BigDecimal countResult = TAl2O3.multiply(VAl2O3.subtract(K.multiply(VCuSO4_I.add(VCuSO4_II)))).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.0000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("Fe2O3")){
			BigDecimal TFe2O3  = getBigDecimal(resultMap.get("TFe2O3"));
			BigDecimal VF2O3  = getBigDecimal(resultMap.get("VF2O3"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult = TFe2O3.multiply(VF2O3).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.0000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("CaO")){
			BigDecimal TCaO  = getBigDecimal(resultMap.get("TCaO"));
			BigDecimal VCaO  = getBigDecimal(resultMap.get("VCaO"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult = TCaO.multiply(VCaO).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("MgO")){
			BigDecimal TMgO  = getBigDecimal(resultMap.get("TMgO"));
			BigDecimal VMgO  = getBigDecimal(resultMap.get("VMgO"));
			BigDecimal VCaO  = getBigDecimal(resultMap.get("VCaO"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult = TMgO.multiply(VMgO.subtract(VCaO)).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.0000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("TiO2")){
			BigDecimal TTiO2  = getBigDecimal(resultMap.get("TTiO2"));
			BigDecimal VCuSO4_II  = getBigDecimal(resultMap.get("VCuSO4(II)"));
			BigDecimal K  = getBigDecimal(resultMap.get("K"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult = TTiO2.multiply(VCuSO4_II).multiply(K).divide(m);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("重量法_SO3")){
			BigDecimal m2  = getBigDecimal(resultMap.get("烧重m2"));
			BigDecimal m1  = getBigDecimal(resultMap.get("埚重m1"));
			BigDecimal m3  = getBigDecimal(resultMap.get("空白m3"));
			BigDecimal m = getBigDecimal(resultMap.get("样重m"));
			//计算的核心公式结果
			BigDecimal countResult =  m2.subtract(m1).subtract(m3).divide(m).multiply(new BigDecimal(0.343));
			endResult = countResult.multiply(param100);
		}else if(name.equals("K2O")){
			BigDecimal CK2O  = getBigDecimal(resultMap.get("CK2O"));
			BigDecimal V  = getBigDecimal(resultMap.get("V"));
			BigDecimal AK2O  = getBigDecimal(resultMap.get("AK2O"));
			BigDecimal mR20 = getBigDecimal(resultMap.get("样重R2O"));
			//计算的核心公式结果
			BigDecimal countResult = CK2O.multiply(V).multiply(AK2O).divide(mR20).divide(param10000);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("Na2O")){
			BigDecimal CNa2O  = getBigDecimal(resultMap.get("CNa2O"));
			BigDecimal V  = getBigDecimal(resultMap.get("V"));
			BigDecimal ANa2O  = getBigDecimal(resultMap.get("ANa2O"));
			BigDecimal mR20 = getBigDecimal(resultMap.get("样重R2O"));
			//计算的核心公式结果
			BigDecimal countResult = CNa2O.multiply(V).multiply(ANa2O).divide(mR20).divide(param10000);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("MnO")){
			BigDecimal CMnO  = getBigDecimal(resultMap.get("CMnO"));
			BigDecimal V  = getBigDecimal(resultMap.get("V"));
			BigDecimal AMnO  = getBigDecimal(resultMap.get("AMnO"));
			BigDecimal mR20 = getBigDecimal(resultMap.get("样重R2O"));
			//计算的核心公式结果
			BigDecimal countResult = CMnO.multiply(V).multiply(AMnO).divide(mR20).divide(param10000);
			BigDecimal result1 = countResult.multiply(param100);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(2,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("Cl")){
			BigDecimal T  = getBigDecimal(resultMap.get("T"));
			BigDecimal V2  = getBigDecimal(resultMap.get("V2"));
			BigDecimal V1  = getBigDecimal(resultMap.get("V1"));
			BigDecimal mCl = getBigDecimal(resultMap.get("样重Cl"));
			//计算的核心公式结果
			BigDecimal countResult = T.multiply(V2.subtract(V1)).divide(mCl).divide(param10);
			BigDecimal result1 = countResult.multiply(param1000);
			int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
			if((resultInt1&1) == 1){  //如果结果为奇数
				endResult =  countResult.setScale(3,BigDecimal.ROUND_HALF_UP);
			}else{
				endResult = countResult.subtract(new BigDecimal(0.00000000001)).setScale(3,BigDecimal.ROUND_HALF_UP);
			}
		}else if(name.equals("Mad1")){
			BigDecimal YZ_M1  = getBigDecimal(resultMap.get("样重M1"));
			BigDecimal MZ_M1  = getBigDecimal(resultMap.get("⽫重M1"));
			BigDecimal SZ_M1  = getBigDecimal(resultMap.get("烧重M1"));
			//计算的核心公式结果
			BigDecimal countResult = YZ_M1.add(MZ_M1).subtract(SZ_M1).divide(YZ_M1);
			endResult = countResult.multiply(param100);
		}else if(name.equals("Mad2")){
			BigDecimal YZ_M2  = getBigDecimal(resultMap.get("样重M2"));
			BigDecimal MZ_M2  = getBigDecimal(resultMap.get("⽫重M2"));
			BigDecimal SZ_M2  = getBigDecimal(resultMap.get("烧重M2"));
			//计算的核心公式结果
			BigDecimal countResult = YZ_M2.add(MZ_M2).subtract(SZ_M2).divide(YZ_M2);
			endResult = countResult.multiply(param100);
		}else if(name.equals("Aad1")){
			BigDecimal SZ_A1  = getBigDecimal(resultMap.get("烧重A1"));
			BigDecimal MZ_A1  = getBigDecimal(resultMap.get("⽫重A1"));
			BigDecimal YZ_A1  = getBigDecimal(resultMap.get("样重A1"));
			//计算的核心公式结果
			BigDecimal countResult = SZ_A1.subtract(MZ_A1).divide(YZ_A1);
			endResult = countResult.multiply(param100);
		}else if(name.equals("Aad2")){
			BigDecimal SZ_A2  = getBigDecimal(resultMap.get("烧重A2"));
			BigDecimal MZ_A2  = getBigDecimal(resultMap.get("⽫重A2"));
			BigDecimal YZ_A2  = getBigDecimal(resultMap.get("样重A2"));
			//计算的核心公式结果
			BigDecimal countResult = SZ_A2.subtract(MZ_A2).divide(YZ_A2);
			endResult = countResult.multiply(param100);
		}

		return endResult;
	}

	/**
	 * 求SO3
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countSO3(Map<String, Object> countMap) {
		BigDecimal weightSO3 = getBigDecimal(countMap.get("重量法_SO3"));
		BigDecimal displaySO3= getBigDecimal(countMap.get("显示值"));
		BigDecimal mSO3= getBigDecimal(countMap.get("样重SO3"));
		BigDecimal elseResult = displaySO3.multiply(new BigDecimal(2.5));
		BigDecimal countResult = weightSO3.compareTo(new BigDecimal(0))==1?
				weightSO3:elseResult;
		BigDecimal result1 = countResult.multiply(param1000);
		int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
		BigDecimal count = null;
		if((resultInt1&1) == 1){  //如果结果为奇数
			BigDecimal jsResult =  weightSO3.compareTo(new BigDecimal(0))==1?
					weightSO3:elseResult.multiply(mSO3).divide(mSO3);
			count =  jsResult.setScale(3,BigDecimal.ROUND_HALF_UP);
		}else{
			BigDecimal osResult =  weightSO3.compareTo(new BigDecimal(0))==1?
					weightSO3:elseResult.multiply(mSO3).divide(mSO3);
			count = osResult.subtract(new BigDecimal(0.00000000001)).setScale(3,BigDecimal.ROUND_HALF_UP);
		}
		return count;
	}

	/**
	 *  求合量  L.O.I+SiO2+(Al2O3+TiO2)+Al2O3+Fe2O3+CaO+MgO+TiO2+K2O+Na2O+MnO+SO3
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countHL(Map<String, Object> countMap) {
		List<String> list = new ArrayList<>();
		list.add(countMap.get("L.O.I").toString());
		list.add(countMap.get("SiO2").toString());
		list.add(countMap.get("Al2O3+TiO2").toString());
		list.add(countMap.get("Al2O3").toString());
		list.add(countMap.get("Fe2O3").toString());
		list.add(countMap.get("CaO").toString());
		list.add(countMap.get("MgO").toString());
		list.add(countMap.get("TiO2").toString());
		list.add(countMap.get("K2O").toString());
		list.add(countMap.get("Na2O").toString());
		list.add(countMap.get("MnO").toString());
		list.add(countMap.get("SO3").toString());
		BigDecimal count=new BigDecimal(0);
		for(String s:list) {
			count = count.add(new BigDecimal(s));
		}
		return count;
	}

	/**
	 * 求KH (CaO - 1.65*{Al2O3+TiO2} * Fe2O3) / 2.8 / SiO2
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countKH(Map<String, Object> countMap) {
		BigDecimal CaO = getBigDecimal(countMap.get("Cao"));
		BigDecimal Al2O3_TiO2= getBigDecimal(countMap.get("Al2O3+TiO2"));
		BigDecimal Fe2O3 = getBigDecimal(countMap.get("Fe2O3"));
		BigDecimal SiO2 = getBigDecimal(countMap.get("SiO2"));
		BigDecimal KH =
		CaO.subtract(new BigDecimal(1.65).multiply(Al2O3_TiO2)).subtract(new BigDecimal(0.35).multiply(Fe2O3)).divide(new BigDecimal(2.8)).divide(SiO2);
		return KH;
	}

	/**
	 * 求SM  SiO2/ ({Al2O3+TiO2} + Fe2O3)
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countSM(Map<String, Object> countMap) {
		BigDecimal SiO2 = getBigDecimal(countMap.get("SiO2"));
		BigDecimal Al2O3_TiO2= getBigDecimal(countMap.get("Al2O3+TiO2"));
		BigDecimal Fe2O3 = getBigDecimal(countMap.get("Fe2O3"));
		BigDecimal SM = SiO2.divide(Al2O3_TiO2.add(Fe2O3));
		return SM;
	}

	/**
	 * 求AM  「Al2O3+TiO2」/ Fe2O3
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countAM(Map<String, Object> countMap) {
		BigDecimal Al2O3_TiO2= getBigDecimal(countMap.get("Al2O3+TiO2"));
		BigDecimal Fe2O3 = getBigDecimal(countMap.get("Fe2O3"));
		BigDecimal AM = Al2O3_TiO2.divide(Fe2O3);
		return AM;
	}

	/**
	 * 求易烧性等级
	 * @param resultMap 输入集合
	 * @param countMap  输出集合
	 * @return
	 */
	public static String countBurnupLevel(Map<String, Object> resultMap, Map<String, Object> countMap) {
		BigDecimal fCaO_1450  = getBigDecimal(resultMap.get("fCaO_1450"));
		BigDecimal KH= getBigDecimal(countMap.get("KH"));
		BigDecimal E = new BigDecimal(45).multiply(KH).subtract(new BigDecimal(37.4));
		BigDecimal D = new BigDecimal(35).multiply(KH).subtract(new BigDecimal(29.3));
		BigDecimal C = new BigDecimal(26.25).multiply(KH).subtract(new BigDecimal(22.15));
		BigDecimal B = new BigDecimal(18.75).multiply(KH).subtract(new BigDecimal(16));
		if(fCaO_1450.compareTo(E) == 1){
			return "E";
		}else if (fCaO_1450.compareTo(D) == 1){
			return "D";
		}else if (fCaO_1450.compareTo(C) == 1){
			return "C";
		}else if (fCaO_1450.compareTo(B) == 1){
			return "B";
		}else{
			return "A";
		}
	}

	/**
	 * 求报出_Mad  （Mad1 + Mad2 ）/ 2
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countBCMad(Map<String, Object> countMap) {
		BigDecimal Mad1= getBigDecimal(countMap.get("Mad1"));
		BigDecimal Mad2= getBigDecimal(countMap.get("Mad2"));
		//计算Mad1 和 Mad2 的平均值
		BigDecimal MadAve = Mad1.add(Mad2).divide(new BigDecimal(2));
		return  MadAve;
	}

	/**
	 * 求Mad  报出_Mad *100 判断奇偶数判断后的结果
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countMad(Map<String, Object> countMap) {
		BigDecimal MadAve= getBigDecimal(countMap.get("报出_Mad"));
		//判断奇偶数的最终值
		BigDecimal result1 = MadAve.multiply(param100);
		int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
		BigDecimal Mad = null;
		if((resultInt1&1) == 1){  //如果结果为奇数
			Mad =  MadAve.setScale(2,BigDecimal.ROUND_HALF_UP);
		}else{
			Mad = MadAve.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
		}
		return  Mad;
	}

	/**
	 * 求报出_Aad  (Aad1 + Aad2)/2
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countBCAad(Map<String, Object> countMap) {
		BigDecimal Aad1= getBigDecimal(countMap.get("Aad1"));
		BigDecimal Aad2= getBigDecimal(countMap.get("Aad2"));
		//计算Mad1 和 Mad2 的平均值
		BigDecimal AadAve = Aad1.add(Aad2).divide(new BigDecimal(2));
		return  AadAve;
	}

	/**
	 * 求Aad  报出_Aad *100 判断奇偶数判断后的结果
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countAad(Map<String, Object> countMap) {
		BigDecimal AadAve= getBigDecimal(countMap.get("报出_Aad"));
		//判断奇偶数的最终值
		BigDecimal result1 = AadAve.multiply(param100);
		int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
		BigDecimal Aad = null;
		if((resultInt1&1) == 1){  //如果结果为奇数
			Aad =  AadAve.setScale(2,BigDecimal.ROUND_HALF_UP);
		}else{
			Aad = AadAve.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
		}
		return  Aad;
	}

	/**
	 * 求Vad1
	 * @param resultMap
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countVad1(Map<String, Object> resultMap, Map<String, Object> countMap) {
		BigDecimal YZ_V1  = getBigDecimal(resultMap.get("样重V1"));
		BigDecimal MZ_V1  = getBigDecimal(resultMap.get("皿重V1"));
		BigDecimal SZ_V1  = getBigDecimal(resultMap.get("烧重V1"));
		BigDecimal BC_Mad  = getBigDecimal(resultMap.get("报出_Mad"));
		//计算的核心公式结果
		BigDecimal countResult = YZ_V1.add(MZ_V1).subtract(SZ_V1).divide(YZ_V1);
		BigDecimal count = countResult.multiply(param100).subtract(BC_Mad);
		return  count;
	}

	/**
	 * 求Vad2
	 * @param resultMap
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countVad2(Map<String, Object> resultMap, Map<String, Object> countMap) {
		BigDecimal YZ_V2  = getBigDecimal(resultMap.get("样重V2"));
		BigDecimal MZ_V2  = getBigDecimal(resultMap.get("皿重V2"));
		BigDecimal SZ_V2  = getBigDecimal(resultMap.get("烧重V2"));
		BigDecimal BC_Mad  = getBigDecimal(countMap.get("报出_Mad"));
		//计算的核心公式结果
		BigDecimal countResult = YZ_V2.add(MZ_V2).subtract(SZ_V2).divide(YZ_V2);
		BigDecimal count = countResult.multiply(param100).subtract(BC_Mad);
		return  count;
	}

	/**
	 * 求报出_Vad  (Vad1+ Vad2)/2
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countBCVad(Map<String, Object> countMap) {
		BigDecimal Vad1= getBigDecimal(countMap.get("Vad1"));
		BigDecimal Vad2= getBigDecimal(countMap.get("Vad2"));
		//计算Vad1 和 Vad2 的平均值
		BigDecimal VadAve = Vad1.add(Vad2).divide(new BigDecimal(2));
		return  VadAve;
	}

	/**
	 * 求Vad  报出_Vad *100 判断奇偶数判断后的结果
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countVad(Map<String, Object> countMap) {
		BigDecimal VadAve= getBigDecimal(countMap.get("报出_Vad"));
		//判断奇偶数的最终值
		BigDecimal result1 = VadAve.multiply(param100);
		int resultInt1  = result1.setScale( 0, BigDecimal.ROUND_DOWN ).intValue();
		BigDecimal Vad = null;
		if((resultInt1&1) == 1){  //如果结果为奇数
			Vad =  VadAve.setScale(2,BigDecimal.ROUND_HALF_UP);
		}else{
			Vad = VadAve.subtract(new BigDecimal(0.00000000001)).setScale(2,BigDecimal.ROUND_HALF_UP);
		}
		return  Vad;
	}

	/**
	 * 求Vdaf  Vad *100/(100-Mad-Aad)
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countVdaf(Map<String, Object> countMap) {
		BigDecimal Vad= getBigDecimal(countMap.get("Vad"));
		BigDecimal Mad= getBigDecimal(countMap.get("Mad"));
		BigDecimal Aad= getBigDecimal(countMap.get("Aad"));
		BigDecimal Vdaf = Vad.multiply(param100).multiply(param100.subtract(Mad).subtract(Aad));
		return  Vdaf;
	}

	/**
	 * 求Vdaf  Vad *100/(100-Mad-Aad)
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countK1_(Map<String, Object> countMap) {
		BigDecimal Vdaf= getBigDecimal(countMap.get("Vdaf"));
		BigDecimal K1_=
		Vdaf.compareTo(new BigDecimal(62))==1?new BigDecimal(61.5):(
				Vdaf.compareTo(new BigDecimal(56))==1?new BigDecimal(63):(
				Vdaf.compareTo(new BigDecimal(49))==1?new BigDecimal(65):(
				Vdaf.compareTo(new BigDecimal(45))==1?new BigDecimal(67):(
				Vdaf.compareTo(new BigDecimal(37))==1?new BigDecimal(68.5):
				new BigDecimal(0)
				))));
		return  K1_;
	}

	/**
	 * 求K  if 焦渣特征=1 JZB1  if 焦渣特征=1.5 JZB1_5 f 焦渣特征=2 JZB2
	 * @param resultMap
	 * @param countMap
	 * @return
	 */
	public static String countK(Map<String, Object> resultMap, Map<String, Object> countMap) {
		String JZTZ= resultMap.get("焦渣特征").toString();
		BigDecimal Vdaf= getBigDecimal(countMap.get("Vdaf"));
		//获取焦渣特征序号对应的值
		Map<String, String>  map = JZTZCountUtil.getJZTZ(Vdaf);
		String count = "";
		if(JZTZ.equals("1")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("1.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("2")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("2.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("3")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("3.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("4")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("4.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("5.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("6")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("6.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("7")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("7.5")){
			count = map.get(JZTZ);
		}else if(JZTZ.equals("8")){
			count = map.get(JZTZ);
		}else{
			count = "false";
		}

		return  count;
	}

	/**
	 * 求Vdaf判断  Vdaf<35?1:0
	 * @param countMap
	 * @return
	 */
	public static Integer countVdafPD(Map<String, Object> countMap) {
		BigDecimal Vdaf= getBigDecimal(countMap.get("Vdaf"));
		Integer VdafPD = Vdaf.compareTo(new BigDecimal(35))==-1?1:0;
		return  VdafPD;
	}

	/**
	 * 求Mad判断  Mad>3?1:0
	 * @param countMap
	 * @return
	 */
	public static Integer countMadPD(Map<String, Object> countMap) {
		BigDecimal BC_Mad= getBigDecimal(countMap.get("报出_Mad"));
		Integer MadPD = BC_Mad.compareTo(new BigDecimal(3))==1?1:0;
		return  MadPD;
	}

	/**
	 * 求40Mad 0或1     Vdaf判断* Mad判断
	 * @param countMap
	 * @return
	 */
	public static BigDecimal count40Mad0Or1(Map<String, Object> countMap) {
		BigDecimal VdafPD= getBigDecimal(countMap.get("Vdaf判断"));
		BigDecimal MadPD= getBigDecimal(countMap.get("Mad判断"));
		BigDecimal count = VdafPD.multiply(MadPD);
		return  count;
	}

	/**
	 * 求40Mad 0或1     Vdaf判断* Mad判断
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countAd(Map<String, Object> countMap) {
		BigDecimal BC_Aad= getBigDecimal(countMap.get("报出_Aad"));
		BigDecimal BC_Mad= getBigDecimal(countMap.get("报出_Mad"));
		BigDecimal count = BC_Aad.multiply(param100).divide(param100.subtract(BC_Mad));
		return  count;
	}

	/**
	 * 求Ad系数0.1或0   if Ad>20 0.1 else 0
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countAdXS(Map<String, Object> countMap) {
		BigDecimal Ad= getBigDecimal(countMap.get("Ad"));
		BigDecimal count =
		Ad.compareTo(new BigDecimal(20))==1?new BigDecimal(0.1):new BigDecimal(0);
		return  count;
	}

	public static BigDecimal countVdafXS(Map<String, Object> countMap) {
		BigDecimal Ad= getBigDecimal(countMap.get("Ad"));
		BigDecimal count =
				Ad.compareTo(new BigDecimal(30))==1?new BigDecimal(0.8):(
				Ad.compareTo(new BigDecimal(25))==1?new BigDecimal(0.85):(
				Ad.compareTo(new BigDecimal(20))==1?new BigDecimal(0.95):(
				Ad.compareTo(new BigDecimal(15))==1?new BigDecimal(0.8):(
				Ad.compareTo(new BigDecimal(10))==1?new BigDecimal(0.9):
				new BigDecimal(0.95)
				))));
		return  count;
	}

	/**
	 * 求V，daf  [Vdaf系数0.8~0.95]* Vdaf- [Ad系数0.1或0]*Ad
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countV_daf(Map<String, Object> countMap) {
		BigDecimal VdafXS= getBigDecimal(countMap.get("Vdaf系数0.8~0.95"));
		BigDecimal Vdaf= getBigDecimal(countMap.get("Vdaf"));
		BigDecimal AdXS= getBigDecimal(countMap.get("Ad系数0.1或0"));
		BigDecimal Ad= getBigDecimal(countMap.get("Ad"));
		BigDecimal count = VdafXS.multiply(Vdaf).subtract(AdXS.multiply(Ad));
		return  count;
	}

	/**
	 * 求K0，  if V，daf >8 8500 if V，daf >5.5 8400 if V，daf >3 8300  else 8200
	 * @param countMap
	 * @return
	 */
	public static BigDecimal countK0_(Map<String, Object> countMap) {
		BigDecimal V_daf= getBigDecimal(countMap.get("V,daf"));
		BigDecimal count =
				V_daf.compareTo(new BigDecimal(8))==1?new BigDecimal(8500):(
				V_daf.compareTo(new BigDecimal(5.5))==1?new BigDecimal(8400):(
				V_daf.compareTo(new BigDecimal(3))==1?new BigDecimal(8300):
				new BigDecimal(8200)
				));
		return  count;
	}


}












